# Multiple Luminosity level image Maker 8 & 16bit Version
# Ver. 0.02
# add making single image function
# (c)2022 Ohnishi, Yasuo

from ij import IJ, ImagePlus
from ij import WindowManager as WM
from ij.gui import GenericDialog, ImagePanel
from ij.io import DirectoryChooser, OpenDialog
# from ij.gui import GenericDialog
from ij import plugin
from ij.plugin import ChannelSplitter, Duplicator
from ij.plugin import RGBStackMerge, RGBStackConverter
from ij.plugin.filter import ThresholdToSelection
from ij.process import ImageConverter, ImageProcessor
from ij.process import ByteProcessor
from loci.plugins import BF
from java.awt.event import AdjustmentListener, ItemListener
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')

#---Convert Value in 8, 16, 32bit mode
def convValue(Value, Mode):
#     Converting Luminance Value each in 8, 16, 32bit mode
        if Mode == "16-bit":
                Value = Value * 256
        elif Mode == "32-bit":
                Value = float(Value) / 255.0
        else:
                Value = Value
        return Value

#--------------------------------------------
def luminosity_mask_maker(imp1, lighten, interval, sequence, Mode, NegaInvert, Show, chName, initialV):
# imp1: Grayscale image
# lighten: make lighten images (True or False)
# sequence number of image making
# interval: interval of L values
# nImages: number of making images
# NegaInvert: Nega invert True or False
# Close Windows
#    IJ.run(imp1, "Conversions...", "weighted")
    if Show == "Yes":
        imp1.show()
# Make Grayscale Mask Image Files
    subtract_value = initialV + sequence * interval
    V1 = str(subtract_value) # for threshold expression in file name
    V2 = str(convValue(subtract_value, Mode)) # for threshold expression in file name
    image_imp = Duplicator().run(imp1)
    if lighten == False: # make darken images
        IJ.run(image_imp, "Subtract...","value=" + V2)
        V1 = "-" + V1
    else: # make lighten images
        IJ.run(image_imp, "Add...","value=" + V2)
        V1 = "+" + V1
    if NegaInvert == True: # for making inverted Negative mask
        """
        image_imp.getProcessor().invert()
        """
        IJ.run(image_imp, "Invert","")
        V1 = V1 + "_Nega"
    else: # for making positive mask
        V1 = V1 + "_Posi"
    if Show == "Yes":
        image_imp.show()  # Show the subtracted image in window
    IJ.saveAsTiff(image_imp, folder + filename + "_" + chName + "_" + V1  + ". tif") # Save the Subtracted image x 4 x transparency factor
    if Show == "Yes":
        image_imp.hide()
    image_imp.close()
#   save grayscale mask

#-------------------------------------------
### RGB Manipulation 
def RGBmanipulation(imp0, folder,filename, parm, colorCh, stackSize):
#  input Parameters
    if parm is not None: # if parm is not null, then get paraneters...
        Mode, lighten, interval, nImages, Show, initialV = parm 
#     Lighten: make lighten images?
#     interval: interval value for making multiple images
#     nImages:  number of making images
#     Mode: Image Format 16-bit or 8-bit
#     Show: show working windows or not
#            Windows Showing Mode
#     makeSingle: making single image?
#     valueSingle: subtract or add value for single image
        imp0.show()
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in impsCh: # it needs to convert Grayscale image
        if Mode == "8-bit":
            ImageConverter(i).convertToGray8()
        elif Mode == "16-bit":
            ImageConverter(i).convertToGray16()
        else:
            ImageConverter(i).convertToGray32()
    if stackSize !=1: # if imp0 is not monochrome
        """ R ch.""" 
        impR =  impsCh[0]
        """ G ch.""" 
        impG =  impsCh[1]
        """ B ch. & B x 0.3""" 
        impB =  impsCh[2]
# Show Windows
#    imp0.close()
    if Show == "Yes":
        if stackSize !=1: # if imp0 is not monochrome
            impR.show()
            impG.show()
            impB.show()
        imp0.hide()
    if colorCh == "Blue":
        imp1 = Duplicator().run(impB)
        chName0 = "B"
    elif colorCh == "Red":
        imp1 = Duplicator().run(impR)
        chName0 = "R"
    elif colorCh == "Green":
        imp1 = Duplicator().run(impG)
        chName0 = "G"
    else: #Lightness or Luminosity
        imp1 = Duplicator().run(imp0)
        chName0 = "L"
        if colorCh == "Luminosity":
            chName0 = "Lm"
            IJ.run(imp1, "Conversions...", "weighted")
        if Mode == "16-bit":
            ImageConverter(imp1).convertToGray16()
        elif Mode == "8-bit":
            ImageConverter(imp1).convertToGray8()
        imp1.getProcessor().setLut(None)

#-----------------------------------
# Making Darken / Lighten Images
    for i in range(nImages):
#       i: sequence of making images
        NegaInvert = False
        luminosity_mask_maker(imp1, lighten, interval, i, Mode, NegaInvert, Show, chName0, initialV)
# if imp0 is not monochrome
    if stackSize != 1:
        impR.close()
        impG.close()
        impB.close()
    imp0.close()
#
    if Show == "Yes":
        imp1.hide()
    imp1.close()
#   End of def RGBmanipulation 
#


def parameter_dialog(Mode, C0, C1):
    gd = GenericDialog(C0 + "Multiple L level Image Maker / input parameters")
    gd.addCheckbox("Make Lighten images", False)
    gd.addNumericField("Inital subtract (or add) value", 25.0, 1)
    gd.addNumericField("Level interval value", 25.0, 1)
    gd.addNumericField("Number of making images", 6, 0)
    gd.addMessage("For making single image, set Number of images to 1 ")
    showWin = ["Yes","No"]
    gd.addChoice("Show working windows?", showWin, showWin[0])

    gd.showDialog()

    if gd.wasCanceled():
        Mode = "Stop"
        print "User canceled dialog!"
#    else:
#        previewer.threshold()
#    parm_Mode   = "Auto"
    lighten = gd.getNextBoolean()
    initialV =  gd.getNextNumber()
    interval =  gd.getNextNumber()
    nImages =  gd.getNextNumber()
    Show =  gd.getNextChoice()
    return Mode, lighten, interval, int(nImages), str(Show),initialV


def selectChannel(): # dialog of selecting target channel for mask upper & lower threshold
    gd = GenericDialog("Multiple Luminosity level Image Maker / Target Channel Selection")
    scolorCh = ["Luminosity", "Red","Green","Blue"]
    gd.addChoice("Select Target Channel Image", scolorCh, scolorCh[0])
    gd.showDialog()

    colorCh =  gd.getNextChoice()
    if gd.wasCanceled():
        colorCh =  "None"
    return str(colorCh)

#----------main----------
window = WM.getActiveWindow()
Mode = "Auto"
if not window:
    od = OpenDialog("Choose a file")
    folder = od.getDirectory()
    filename = od.getFileName()
    if filename:
        path = folder + filename
        IJ.log(path)
        IJ.log(folder)
        IJ.log(filename)
# Always Open file with Bio-format Plugin
        imps = BF.openImagePlus(folder + filename) # Open file with Bio-format Plugin
        imp0 = imps[0] #imp0: Original Image 
        imp0.show()
    else:
        Mode = "Stop"
else:
    imp0 = IJ.getImage()
    filename = imp0.getTitle()
    folder = IJ.getDirectory("image") # get image's original directory
    folder = IJ.getDirectory("Output_directory")
if Mode == "Auto":
    bitdepth = imp0.getBitDepth()
    if bitdepth == 8:
            Mode = "8-bit"
    elif bitdepth ==16:
            Mode = "16-bit"
    elif bitdepth ==32:
            Mode = "32-bit"
    else:
            Mode = "16-bit"
stackSize = imp0.getStackSize() # imp0 is color or monochrome?
if Mode <> "Stop": # if user did not cancel execute process
    
    if stackSize == 1: #imp0 is monochrome
        colorCh = "Luminosity"
    else:  #imp0 is not monochrome
        colorCh = selectChannel()
        """ select target channel for deciding upper & lower threshold """
    if colorCh == "Blue":
        C0 = "Blue"
        C1 = "Blue inverted"
    elif colorCh == "Red":
        C0 = "Red"
        C1 = "Red inverted"
    elif colorCh == "Green":
        C0 = "Green"
        C1 = "Green inverted"
    elif colorCh == "Luminosity":
        C0 = "Luminosity Positive"
        C1 = "Luminosity Negative"
    elif colorCh == "Lightness":
        C0 = "Lightness Positive"
        C1 = "Lightness Negative"

    if colorCh != "None": # if process is not canceled
        imp0.hide()
        parm = parameter_dialog(Mode, C0, C1)
    else: # if process is  canceled
        parm = []
        parm.append("Stop")
        imp0.hide()
        imp0.close()
    
    if parm[0] <> "Stop":
        RGBmanipulation(imp0, folder, filename, parm, colorCh, stackSize)
else:
    IJ.log("User canceled")
IJ.log("Finish")
